@@ -4,7 +4,7 @@ from abc import ABC, abstractmethod
 import numpy as np
 
 
-class CtrlClass(ABC):
+class ControlClass(ABC):
     """Basic Controller class to implement custom controllers."""
 
     @abstractmethod
