@@ -2,16 +2,18 @@
 from __future__ import annotations
 
 import time
+from itertools import repeat
+from typing import Sequence
 
 import numpy as np
 import pybullet as p
 import pybullet_data
 from pybullet_utils import bullet_client
 
-from .abstractions.base_drone import DroneClass
-from .drones.fixedwing import FixedWing
-from .drones.quadx import QuadX
-from .drones.rocket import Rocket
+from .abstractions import DroneClass
+from .drones import FixedWing, QuadX, Rocket
+
+DroneIndex = int
 
 
 class Aviary(bullet_client.BulletClient):
@@ -21,20 +23,13 @@ class Aviary(bullet_client.BulletClient):
         self,
         start_pos: np.ndarray,
         start_orn: np.ndarray,
+        drone_type: str | Sequence[str],
+        drone_type_mappings: None | dict[str, DroneClass] = None,
         render: bool = False,
         physics_hz: int = 240,
-        ctrl_hz: int = 120,
-        drone_type: str = "quadx",
-        drone_model: str = "cf2x",
-        model_dir: None | str = None,
-        use_camera: bool = False,
-        use_gimbal: bool = False,
-        camera_angle_degrees: int = 20,
-        camera_FOV_degrees: int = 90,
-        camera_resolution: tuple[int, int] = (128, 128),
         worldScale: float = 1.0,
+        drone_options: dict | Sequence[dict] = {},
         seed: None | int = None,
-        drone_options: dict = {},
     ):
         """Initializes a PyBullet environment that hosts UAVs and other entities.
 
@@ -44,25 +39,18 @@ class Aviary(bullet_client.BulletClient):
         Args:
             start_pos (np.ndarray): start_pos
             start_orn (np.ndarray): start_orn
+            drone_type (str | Sequence[str]): drone_types
+            drone_type_mappings (None | dict[str, DroneClass]): string to mapping of custom drone classes
             render (bool): render
             physics_hz (int): physics_hz
-            ctrl_hz (int): ctrl_hz
-            drone_type (str): drone_type
-            drone_model (str): drone_model
-            model_dir (None | str): model_dir
-            use_camera (bool): use_camera
-            use_gimbal (bool): use_gimbal
-            camera_angle_degrees (int): camera_angle_degrees
-            camera_FOV_degrees (int): camera_FOV_degrees
-            camera_resolution (tuple[int, int]): camera_resolution
             worldScale (float): worldScale
+            drone_options (dict | Sequence[dict]): drone_options
             seed (None | int): seed
-            drone_options (dict): drone_options
         """
         super().__init__(p.GUI if render else p.DIRECT)
         print("\033[A                             \033[A")
 
-        # assertations
+        # check for starting position and orientation shapes
         assert (
             len(start_pos.shape) == 2
         ), f"start_pos must be shape (n, 3), currently {start_pos.shape}."
@@ -73,37 +61,58 @@ class Aviary(bullet_client.BulletClient):
             start_orn.shape == start_pos.shape
         ), f"start_orn must be same shape as start_pos, currently {start_orn.shape}."
 
-        # define the drone types
-        if drone_type == "quadx":
-            self.drone_constructor = QuadX
-        elif drone_type == "fixedwing":
-            self.drone_constructor = FixedWing
-        elif drone_type == "rocket":
-            self.drone_constructor = Rocket
+        # check to ensure drone type has same number as drones if is list/tuple
+        if isinstance(drone_type, (tuple, list)):
+            assert (
+                len(drone_type) == start_pos.shape[0]
+            ), f"If multiple `drone_types` are used, must have same number of `drone_types` ({len(drone_type)}) as number of drones ({start_pos.shape[0]})."
+        # check to ensure drone type has same number as drones if is list/tuple
+        if isinstance(drone_options, (tuple, list)):
+            assert (
+                len(drone_options) == start_pos.shape[0]
+            ), f"If multiple `drone_options` ({len(drone_options)}) are used, must have same number of `drone_options` as number of drones ({start_pos.shape[0]})."
 
-        # default physics looprate is 240 Hz
-        # do not change because pybullet doesn't like it
-        self.physics_hz = physics_hz
-        self.physics_period = 1.0 / physics_hz
-        self.ctrl_hz = ctrl_hz
-        self.ctrl_period = 1.0 / ctrl_hz
-        self.ctrl_update_ratio = int(physics_hz / ctrl_hz)
-        self.now = time.time()
-
-        # pybullet stuff
+        # constants
+        self.num_drones = start_pos.shape[0]
         self.start_pos = start_pos
         self.start_orn = start_orn
-        self.use_camera = use_camera
-        self.use_gimbal = use_gimbal
-        self.camera_angle = camera_angle_degrees
-        self.camera_FOV = camera_FOV_degrees
-        self.camera_frame_size = camera_resolution
-        self.worldScale = worldScale
 
-        # directories and paths
-        self.model_dir = model_dir
-        self.drone_model = drone_model
-        self.drone_options = drone_options
+        # do not change because pybullet doesn't like it
+        # default physics looprate is 240 Hz
+        self.physics_hz = physics_hz
+        self.physics_period = 1.0 / physics_hz
+
+        # mapping of drone type string to the constructors
+        self.drone_type_mappings = dict()
+        self.drone_type_mappings["quadx"] = QuadX
+        self.drone_type_mappings["fixedwing"] = FixedWing
+        self.drone_type_mappings["rocket"] = Rocket
+        if drone_type_mappings is not None:
+            self.drone_type_mappings = {
+                **self.drone_type_mappings,
+                **drone_type_mappings,
+            }
+
+        # store all drone types
+        if isinstance(drone_type, (tuple, list)):
+            assert all(
+                dt in self.drone_type_mappings for dt in drone_type
+            ), f"One of types in `drone_type` {drone_type} is not amongst known types {self.drone_type_mappings.keys()}."
+            self.drone_type = drone_type
+        else:
+            assert (
+                drone_type in self.drone_type_mappings
+            ), f"Can't find `drone_type` {drone_type} amongst known types {self.drone_type_mappings.keys()}."
+            self.drone_type = repeat(drone_type)
+
+        # store the drone options
+        if isinstance(drone_options, (tuple, list)):
+            self.drone_options = drone_options
+        else:
+            self.drone_options = repeat(drone_options)
+
+        # set the world scale and directories
+        self.worldScale = worldScale
         self.setAdditionalSearchPath(pybullet_data.getDataPath())
 
         # render
@@ -131,38 +140,40 @@ class Aviary(bullet_client.BulletClient):
             cameraPitch=-30,
             cameraTargetPosition=[0, 0, 1],
         )
-        if not self.use_camera:
-            self.configureDebugVisualizer(p.COV_ENABLE_GUI, 0)
 
         # define new RNG
         self.np_random = np.random.RandomState(seed=seed)
 
-        """ CONSTRUCT THE WORLD """
+        # construct the world
         self.planeId = self.loadURDF(
             "plane.urdf", useFixedBase=True, globalScaling=self.worldScale
         )
 
         # spawn drones
         self.drones: list[DroneClass] = []
-        for start_pos, start_orn in zip(self.start_pos, self.start_orn):
+        for start_pos, start_orn, drone_type, drone_options in zip(
+            self.start_pos,
+            self.start_orn,
+            self.drone_type,
+            self.drone_options,
+        ):
             self.drones.append(
-                self.drone_constructor(
+                self.drone_type_mappings[drone_type](
                     self,
                     start_pos=start_pos,
                     start_orn=start_orn,
-                    ctrl_hz=self.ctrl_hz,
                     physics_hz=self.physics_hz,
-                    drone_model=self.drone_model,
-                    model_dir=self.model_dir,
-                    use_camera=self.use_camera,
-                    use_gimbal=self.use_gimbal,
-                    camera_angle_degrees=self.camera_angle,
-                    camera_FOV_degrees=self.camera_FOV,
-                    camera_resolution=self.camera_frame_size,
                     np_random=self.np_random,
-                    **self.drone_options,
+                    **drone_options,
                 )
             )
+
+        # constants for tracking how many times to step depending on control hz
+        all_control_hz = [1.0 / drone.control_period for drone in self.drones]
+        self.updates_per_step = int(self.physics_hz / np.min(all_control_hz))
+        self.update_period = 1.0 / np.min(all_control_hz)
+        self.now = time.time()
+
         # arm everything
         self.register_all_new_bodies()
         self.set_armed(True)
@@ -177,19 +188,34 @@ class Aviary(bullet_client.BulletClient):
             (self.getNumBodies(), self.getNumBodies()), dtype=bool
         )
 
-    @property
-    def num_drones(self) -> int:
-        """Returns the number of drones in the environment.
+    def state(self, index: DroneIndex) -> np.ndarray:
+        """Returns the state for the indexed drone.
+
+        Args:
+            index (DRONE_INDEX): index
 
         Returns:
-            int: number of drones in the environment
+            np.ndarray: state
         """
-        return len(self.drones)
+        return self.drones[index].state
+
+    def aux_state(self, index: DroneIndex) -> np.ndarray:
+        """Returns the auxiliary state for the indexed drone.
+
+        Args:
+            index (DRONE_INDEX): index
+
+        Returns:
+            np.ndarray: auxiliary state
+        """
+        return self.drones[index].aux_state
 
     @property
-    def states(self) -> np.ndarray:
+    def all_states(self) -> list[np.ndarray]:
         """Returns a list of states for all drones in the environment.
 
+        This function is not very optimized, if you want the state of a single drone, do `state(i)`.
+
         Returns:
             np.ndarray: list of states
         """
@@ -197,14 +223,14 @@ class Aviary(bullet_client.BulletClient):
         for drone in self.drones:
             states.append(drone.state)
 
-        states = np.stack(states, axis=0)
-
         return states
 
     @property
-    def aux_states(self) -> np.ndarray:
+    def all_aux_states(self) -> list[np.ndarray]:
         """Returns a list of auxiliary states for all drones in the environment.
 
+        This function is not very optimized, if you want the aux state of a single drone, do `aux_state(i)`.
+
         Returns:
             np.ndarray: list of auxiliary states
         """
@@ -212,8 +238,6 @@ class Aviary(bullet_client.BulletClient):
         for drone in self.drones:
             aux_states.append(drone.aux_state)
 
-        aux_states = np.stack(aux_states, axis=0)
-
         return aux_states
 
     def print_all_bodies(self):
@@ -258,7 +282,16 @@ class Aviary(bullet_client.BulletClient):
             for drone in self.drones:
                 drone.set_mode(flight_modes)
 
-    def set_setpoints(self, setpoints: np.ndarray):
+    def set_setpoint(self, index: DroneIndex, setpoint: np.ndarray):
+        """Sets the setpoint of one drone in the environment.
+
+        Args:
+            index (DRONE_INDEX): index
+            setpoint (np.ndarray): setpoint
+        """
+        self.drones[index].setpoint = setpoint
+
+    def set_all_setpoints(self, setpoints: np.ndarray):
         """Sets the setpoints of each drone in the environment.
 
         Args:
@@ -275,10 +308,10 @@ class Aviary(bullet_client.BulletClient):
             self.now = time.time()
 
             # sleep to maintain real time factor
-            time.sleep(max(0, self.ctrl_period - elapsed))
+            time.sleep(max(0, self.update_period - elapsed))
 
             # calculate real time factor
-            RTF = self.ctrl_period / (elapsed + 1e-6)
+            RTF = self.update_period / (elapsed + 1e-6)
 
             # handle case where sometimes elapsed becomes 0
             if elapsed != 0.0:
@@ -292,11 +325,16 @@ class Aviary(bullet_client.BulletClient):
         # reset collisions
         self.collision_array &= False
 
-        # update onboard avionics compute
-        [drone.update_avionics() for drone in self.armed_drones]
+        # step the environment enough times for one control loop of the slowest controller
+        physics_steps = 1
+        while physics_steps <= self.updates_per_step:
+            # update onboard avionics conditionally
+            [
+                drone.update_avionics()
+                for drone in self.armed_drones
+                if physics_steps % drone.physics_control_ratio == 0
+            ]
 
-        # step the environment enough times for one control loop
-        for _ in range(self.ctrl_update_ratio):
             # compute physics
             [drone.update_physics() for drone in self.armed_drones]
 
@@ -308,4 +346,6 @@ class Aviary(bullet_client.BulletClient):
                 self.collision_array[collision[1], collision[2]] = True
                 self.collision_array[collision[2], collision[1]] = True
 
+            physics_steps += 1
+
         self.steps += 1
