@@ -7,7 +7,7 @@ import numpy as np
 import yaml
 from pybullet_utils import bullet_client
 
-from ..abstractions.base_controller import CtrlClass
+from ..abstractions.base_controller import ControlClass
 from ..abstractions.base_drone import DroneClass
 from ..abstractions.camera import Camera
 from ..abstractions.motors import Motors
@@ -22,16 +22,16 @@ class QuadX(DroneClass):
         p: bullet_client.BulletClient,
         start_pos: np.ndarray,
         start_orn: np.ndarray,
-        ctrl_hz: int,
-        physics_hz: int,
+        control_hz: int = 120,
+        physics_hz: int = 240,
         drone_model: str = "cf2x",
         model_dir: None | str = None,
+        np_random: None | np.random.RandomState = None,
         use_camera: bool = False,
         use_gimbal: bool = False,
         camera_angle_degrees: int = 20,
         camera_FOV_degrees: int = 90,
         camera_resolution: tuple[int, int] = (128, 128),
-        np_random: None | np.random.RandomState = None,
     ):
         """Creates a drone in the QuadX configuration and handles all relevant control and physics.
 
@@ -39,22 +39,22 @@ class QuadX(DroneClass):
             p (bullet_client.BulletClient): p
             start_pos (np.ndarray): start_pos
             start_orn (np.ndarray): start_orn
-            ctrl_hz (int): ctrl_hz
+            control_hz (int): control_hz
             physics_hz (int): physics_hz
-            model_dir (None | str): model_dir
             drone_model (str): drone_model
+            model_dir (None | str): model_dir
+            np_random (None | np.random.RandomState): np_random
             use_camera (bool): use_camera
             use_gimbal (bool): use_gimbal
             camera_angle_degrees (int): camera_angle_degrees
             camera_FOV_degrees (int): camera_FOV_degrees
             camera_resolution (tuple[int, int]): camera_resolution
-            np_random (None | np.random.RandomState): np_random
         """
         super().__init__(
             p=p,
             start_pos=start_pos,
             start_orn=start_orn,
-            ctrl_hz=ctrl_hz,
+            control_hz=control_hz,
             physics_hz=physics_hz,
             model_dir=model_dir,
             drone_model=drone_model,
@@ -62,8 +62,15 @@ class QuadX(DroneClass):
         )
         """
         DRONE CONTROL
-            motor ids correspond to quadrotor X in PX4, using the ENU convention
-            control commands are in the form of pitch-roll-yaw-thrust
+            motor ids correspond to quadrotor X in PX4,
+            -------------------------
+            |  (cw)2  0(ccw)        |
+            |      \\//           x |
+            |      //\\         y_| |
+            | (ccw)1  3(cw)         |
+            -------------------------
+            using the ENU convention
+            control commands are in the form of roll-pitch-yaw-thrust
         """
 
         # All the params for the drone
@@ -79,10 +86,10 @@ class QuadX(DroneClass):
             thrust_coef = np.array([motor_params["thrust_coef"]] * 4)
             torque_coef = np.array(
                 [
-                    +motor_params["torque_coef"],
-                    +motor_params["torque_coef"],
                     -motor_params["torque_coef"],
                     -motor_params["torque_coef"],
+                    +motor_params["torque_coef"],
+                    +motor_params["torque_coef"],
                 ]
             )
             thrust_unit = np.array(
@@ -115,10 +122,10 @@ class QuadX(DroneClass):
             # motor mapping from command to individual motors
             self.motor_map = np.array(
                 [
+                    [-1.0, -1.0, -1.0, +1.0],
+                    [+1.0, +1.0, -1.0, +1.0],
                     [+1.0, -1.0, +1.0, +1.0],
                     [-1.0, +1.0, +1.0, +1.0],
-                    [+1.0, +1.0, -1.0, +1.0],
-                    [-1.0, -1.0, -1.0, +1.0],
                 ]
             )
 
@@ -154,14 +161,14 @@ class QuadX(DroneClass):
                 ctrl_params["z_pos"]["ki"],
                 ctrl_params["z_pos"]["kd"],
                 ctrl_params["z_pos"]["lim"],
-                self.ctrl_period,
+                self.control_period,
             )
             z_vel_PID = PID(
                 ctrl_params["z_vel"]["kp"],
                 ctrl_params["z_vel"]["ki"],
                 ctrl_params["z_vel"]["kd"],
                 ctrl_params["z_vel"]["lim"],
-                self.ctrl_period,
+                self.control_period,
             )
             self.z_PIDs = [z_vel_PID, z_pos_PID]
             self.PIDs = []
@@ -260,7 +267,7 @@ class QuadX(DroneClass):
                 self.Ki_ang_vel,
                 self.Kd_ang_vel,
                 self.lim_ang_vel,
-                self.ctrl_period,
+                self.control_period,
             )
             self.PIDs = [ang_vel_PID]
         elif mode in [1, 3]:
@@ -269,14 +276,14 @@ class QuadX(DroneClass):
                 self.Ki_ang_vel,
                 self.Kd_ang_vel,
                 self.lim_ang_vel,
-                self.ctrl_period,
+                self.control_period,
             )
             ang_pos_PID = PID(
                 self.Kp_ang_pos,
                 self.Ki_ang_pos,
                 self.Kd_ang_pos,
                 self.lim_ang_pos,
-                self.ctrl_period,
+                self.control_period,
             )
             self.PIDs = [ang_vel_PID, ang_pos_PID]
         elif mode in [4, 5, 6]:
@@ -285,21 +292,21 @@ class QuadX(DroneClass):
                 self.Ki_ang_vel,
                 self.Kd_ang_vel,
                 self.lim_ang_vel,
-                self.ctrl_period,
+                self.control_period,
             )
             ang_pos_PID = PID(
                 self.Kp_ang_pos[:2],
                 self.Ki_ang_pos[:2],
                 self.Kd_ang_pos[:2],
                 self.lim_ang_pos[:2],
-                self.ctrl_period,
+                self.control_period,
             )
             lin_vel_PID = PID(
                 self.Kp_lin_vel,
                 self.Ki_lin_vel,
                 self.Kd_lin_vel,
                 self.lim_lin_vel,
-                self.ctrl_period,
+                self.control_period,
             )
             self.PIDs = [ang_vel_PID, ang_pos_PID, lin_vel_PID]
         elif mode == 7:
@@ -308,28 +315,28 @@ class QuadX(DroneClass):
                 self.Ki_ang_vel,
                 self.Kd_ang_vel,
                 self.lim_ang_vel,
-                self.ctrl_period,
+                self.control_period,
             )
             ang_pos_PID = PID(
                 self.Kp_ang_pos,
                 self.Ki_ang_pos,
                 self.Kd_ang_pos,
                 self.lim_ang_pos,
-                self.ctrl_period,
+                self.control_period,
             )
             lin_vel_PID = PID(
                 self.Kp_lin_vel,
                 self.Ki_lin_vel,
                 self.Kd_lin_vel,
                 self.lim_lin_vel,
-                self.ctrl_period,
+                self.control_period,
             )
             lin_pos_PID = PID(
                 self.Kp_lin_pos,
                 self.Ki_lin_pos,
                 self.Kd_lin_pos,
                 self.lim_lin_pos,
-                self.ctrl_period,
+                self.control_period,
             )
             self.PIDs = [ang_vel_PID, ang_pos_PID, lin_vel_PID, lin_pos_PID]
 
@@ -339,7 +346,7 @@ class QuadX(DroneClass):
     def register_controller(
         self,
         controller_id: int,
-        controller_constructor: type[CtrlClass],
+        controller_constructor: type[ControlClass],
         base_mode: int,
     ):
         """Registers a new controller for the UAV.
@@ -462,8 +469,14 @@ class QuadX(DroneClass):
         if (low := np.min(self.pwm)) < 0.05:
             self.pwm += (1.0 - self.pwm) / (1.0 - low) * (0.05 - low)
 
-    def update_drag(self):
-        """Adds drag to the model, this is not physically correct but only approximation."""
+    def update_physics(self):
+        """Updates the physics of the vehicle."""
+        self.update_state()
+
+        # update the motors
+        self.motors.pwm2forces(self.pwm)
+
+        # simulate drag
         drag_pqr = -self.drag_coef_pqr * (np.array(self.state[0]) ** 2)
         drag_xyz = -self.drag_coef_xyz * (np.array(self.state[2]) ** 2)
 
@@ -474,12 +487,6 @@ class QuadX(DroneClass):
             )
             self.p.applyExternalTorque(self.Id, -1, drag_pqr, self.p.LINK_FRAME)
 
-    def update_physics(self):
-        """Updates the physics of the vehicle."""
-        self.update_state()
-        self.motors.pwm2forces(self.pwm)
-        self.update_drag()
-
     def update_avionics(self):
         """Updates state and control."""
         self.update_control()
